require 'base64'
require 'openssl'

module TimeSensitiveHmac
  class Signature

    attr_reader :secret, :digest

    def initialize(secret, opts={})
      @secret = secret
      @digest = OpenSSL::Digest::Digest.new(
                  opts[:digest] || 'sha256'
                )
    end

    def generate(time, data, context=nil)
      raw = OpenSSL::HMAC.digest(
        digest,
        secret,
        data_from_parts(time, context, data)
      )
      encode(raw)
    end

    def verify(sig, time, data, context=nil, grace_period_in_seconds=0)
      # TODO: grace period
      # Take inspiration from HOTP (RFC 4226) for time intervals:
      # http://tools.ietf.org/html/rfc4226#page-35
      check_sig = generate(time, data, context)
      sig == check_sig
    end

    def verify_now(sig, data, context=nil, grace_period_in_seconds=0)
      verify(sig, Time.now, context, grace_period_in_seconds)
    end

    protected

      def encode(input)
        # encode64 includes a trailing \n.
        input && Base64.encode64(input).strip
      end

      def data_from_parts(time, context, data)
        [time.utc.to_i, encode(context), encode(data)].compact.join(':')
      end

      def normalise_to_time_class(time)
        unless time.is_a? Time
          time = Time.at(time.to_i)
        end
        time
      end
  end
end
